#include <pcap.h>
#include <windows.h>
#include <stdio.h>
#include "gsirc.h"

#pragma comment(lib, "ws2_32.lib")
#pragma comment(lib, "wpcap.lib")

#define GSA_IP "207.38.8.36"

void packet_handler(u_char *param, const struct pcap_pkthdr *header, const u_char *pkt_data);
void parse(char *buf, int length);
void parseout(char *buf, int length);
int active;
FILE *file;
int LOG;

/* 4 bytes IP address */
typedef struct ip_address{
    u_char byte1;
    u_char byte2;
    u_char byte3;
    u_char byte4;
}ip_address;

/* IPv4 header */
typedef struct ip_header{
    u_char  ver_ihl;        // Version (4 bits) + Internet header length (4 bits)
    u_char  tos;            // Type of service 
    u_short tlen;           // Total length 
    u_short identification; // Identification
    u_short flags_fo;       // Flags (3 bits) + Fragment offset (13 bits)
    u_char  ttl;            // Time to live
    u_char  proto;          // Protocol
    u_short crc;            // Header checksum
    ip_address  saddr;      // Source address
    ip_address  daddr;      // Destination address
    u_int   op_pad;         // Option + Padding
}ip_header;

/* TCP header */
typedef struct sniff_tcp {
    u_short th_sport; /* source port */
    u_short th_dport; /* destination port */
    u_long th_seq; /* sequence number */
    u_long th_ack; /* acknowledgement number */
    u_char th_flags;
    #define TH_FIN 0x01
    #define TH_SYN 0x02
    #define TH_RST 0x04
    #define TH_PUSH 0x08
    #define TH_ACK 0x10
    #define TH_URG 0x20
    #define TH_ECE 0x40
    #define TH_CWR 0x80
    #define TH_FLAGS (TH_FIN|TH_SYN|TH_RST|TH_ACK|TH_URG|TH_ECE|TH_CWR)
    u_short th_win; /* window */
    u_short th_sum; /* checksum */
    u_short th_urp; /* urgent pointer */
}sniff_tcp;

int main(int argc, char *argv[])
{
	if(argc != 2) { //usage
		printf("\nGameSpy Arcade IRC Sniffer with Encryption/Decryption\n"
			   "by xoclipse\n"
			   "email: xoclipse@cox.net\n"
			   "web: http://xoclipse.fraghosts.net\n"
			   "usage: [-l] [-o]\n"
			   "-l: log to file and output to screen\n"
			   "-o: output to screen only\n\n");
		exit(1);
	}
	if(argv[1][1] == 'l') {
		LOG = 1;
	}
    pcap_if_t *alldevs;
    pcap_if_t *d;
    int inum;
    int i=0;
    pcap_t *adhandle;
    char errbuf[PCAP_ERRBUF_SIZE];
    
    /* Retrieve the device list */
    if (pcap_findalldevs(&alldevs, errbuf) == -1)
    {
        fprintf(stderr,"Error in pcap_findalldevs: %s\n", errbuf);
        exit(1);
    }
    
    /* Print the list */
    for(d=alldevs; d; d=d->next)
    {
        printf("%d. %s", ++i, d->name);
        if (d->description)
            printf(" (%s)\n", d->description);
        else
            printf(" (No description available)\n");
    }
    
    if(i==0)
    {
        printf("\nNo interfaces found! Make sure WinPcap is installed.\n");
        return -1;
    }
    
    printf("Enter the interface number (1-%d):",i);
    scanf("%d", &inum);
    
    if(inum < 1 || inum > i)
    {
        printf("\nInterface number out of range.\n");
        /* Free the device list */
        pcap_freealldevs(alldevs);
        return -1;
    }
    
    /* Jump to the selected adapter */
    for(d=alldevs, i=0; i< inum-1 ;d=d->next, i++);
    
    /* Open the adapter */
    if ( (adhandle= pcap_open_live(d->name, // name of the device
                             65536,     // portion of the packet to capture. 
                             // 65536 grants that the whole packet will be captured on all the MACs.
                             1,         // promiscuous mode
                             1000,      // read timeout
                             errbuf     // error buffer
                             ) ) == NULL)
    {
        fprintf(stderr,"\nUnable to open the adapter. %s is not supported by WinPcap\n");
        /* Free the device list */
        pcap_freealldevs(alldevs);
        return -1;
    }
    
    printf("\nlistening on %s...\n", d->description);
    
    /* At this point, we don't need any more the device list. Free it */
    pcap_freealldevs(alldevs);
    
    /* start the capture */
	
	if(LOG == 1) {
	time_t rawtime;
    struct tm * timeinfo;

    time ( &rawtime );
    timeinfo = localtime ( &rawtime );
    char filename[100];
    sprintf(filename,"2003-%d-%d.txt", timeinfo->tm_mon+1, timeinfo->tm_mday);
	file = fopen(filename, "a+");
	fputs("Log File Opened\r\n\r\nCapture Started\r\n", file);
	}
    pcap_loop(adhandle, 0, packet_handler, NULL);
	if(LOG==1) {
	    fclose(file);
	}
    return 0;
}


/* Callback function invoked by libpcap for every incoming packet */
void packet_handler(u_char *param, const struct pcap_pkthdr *header, const u_char *pkt_data)
{
    struct tm *ltime;
    char timestr[16];
	ip_header *ih;
	sniff_tcp *tcp;
	int len;
	char *data;
    char ip[32];
	char ip1[32];
    /* convert the timestamp to readable format */
    ltime=localtime(&header->ts.tv_sec);
    strftime( timestr, sizeof timestr, "%H:%M:%S", ltime);
    
    //printf("%s,%.6d len:%d\n", timestr, header->ts.tv_usec, header->len);


	/* retireve the position of the ip header */
    ih = (ip_header *) (pkt_data +14); //length of ethernet header
	len = sprintf(ip, "%d.%d.%d.%d", ih->saddr.byte1, ih->saddr.byte2, ih->saddr.byte3, ih->saddr.byte4); 
	ip[len] = 0;
	len = sprintf(ip1, "%d.%d.%d.%d", ih->daddr.byte1, ih->daddr.byte2, ih->daddr.byte3, ih->daddr.byte4); 
	ip1[len] = 0;
	tcp = (sniff_tcp *) (pkt_data +34);
	data = (char*)(pkt_data + 54);
	if(strncmp(ip, GSA_IP, 11) == 0) //gamespy server -> client
	{
		if(ntohs(tcp->th_sport) == 6667) //right gamespy service
		{
			//printf("%s:%d -> %s:%d Length: %d\n", ip,ntohs(tcp->th_sport), ip1,ntohs(tcp->th_dport),(ntohs(ih->tlen)-40));
			if((ntohs(ih->tlen)-40) > 0) {
				parse(data, (ntohs(ih->tlen)-40));
			}
		}
	}
	else if(strncmp(ip1, GSA_IP, 11) == 0)
	{
		if(ntohs(tcp->th_dport) == 6667) //right gamespy service
		{
			//printf("%s:%d -> %s:%d Length: %d\n", ip,ntohs(tcp->th_sport), ip1,ntohs(tcp->th_dport),(ntohs(ih->tlen)-40));
			if((ntohs(ih->tlen)-40) > 0) {
				parseout(data, (ntohs(ih->tlen)-40));
			}
		}
	}
}
//--------------------------------------------------------------
void parse(char *buf, int length)
{
	int ret;
	if(strncmp(buf, ":s 705",6) == 0) { //here's our keys
		buf += 9;
		memcpy(key, buf, 16);
		memcpy(key1, buf+17, 16);
		initbuffer();
		//char *id = encrypt("USRIP\r\n", 7);
		//send(sockfd, id, 7, 0);
		printf("Detected Login in encrypted mode...\n\n");
if(LOG == 1) {
		fputs("Detected new Login in encrypted mode...\r\n\r\n\r\n", file);
}
	}
	else
	{
		char *stuff = decrypt(buf, length);
				printf("GameSpy Server -> Client: \n");
		if(LOG == 1) {
		fputs("GameSpy Server -> Client: \r\n", file);
		}
		for(int x = 0; x < length;x++) printf("%c", (unsigned char)stuff[x]);
		printf("\n");
		stuff[length] = 0;
if(LOG == 1) {
		fputs(stuff, file);
		fputs("\r\n", file);
}
	}
	
}
//----------------------------------------------------------------
void parseout(char *buf, int length)
{
	char *stuff = encrypt(buf, length);
	printf("Client -> GameSpy Server: \n");
	if(LOG == 1) {
	fputs("Client -> GameSpy Server: \r\n", file);
	}
	
	for(int x = 0; x < length;x++) printf("%c", (unsigned char)stuff[x]);
	printf("\n");
	stuff[length] = 0;
if(LOG == 1) {
	fputs(stuff, file);
	fputs("\r\n", file);
}
}
//-----------------------------------------------------------------
